@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: client.c,v 1.156.2.16 2005/03/31 06:07:10 priikone Exp $ */
+/* $Id: client.c,v 1.156.2.19 2006/08/08 18:58:37 coffee Exp $ */
 
 #include "silcincludes.h"
 #include "silcclient.h"
@@ -254,11 +254,11 @@ silc_client_add_connection(SilcClient client,
   conn->remote_port = port;
   conn->context = context;
   conn->internal->client_cache =
-    silc_idcache_alloc(0, SILC_ID_CLIENT, NULL, FALSE, TRUE);
+    silc_idcache_alloc(0, SILC_ID_CLIENT, NULL, NULL, FALSE, TRUE);
   conn->internal->channel_cache = silc_idcache_alloc(0, SILC_ID_CHANNEL, NULL,
-						     FALSE, TRUE);
+						     NULL, FALSE, TRUE);
   conn->internal->server_cache = silc_idcache_alloc(0, SILC_ID_SERVER, NULL,
-						    FALSE, TRUE);
+						    NULL, FALSE, TRUE);
   conn->internal->pending_commands = silc_dlist_init();
   conn->internal->ftp_sessions = silc_dlist_init();
 
